/*
 * (c) Copyright 1999-2008 PaperCut Software Pty. Ltd.
 * $Id$
 */
package biz.papercut.pcng.plugin;

import java.util.Map;

/**
 * Plugins can implement this interface to dynamically add extra money to accounts when current balance is lower than
 * what is required.  e.g. use to add $1.25 to an account when the job costs $1.25 and they have a balance of $0.
 * 
 * NOTE: New implementations of topup on demand should implement TopupAccountOnDemandPlugin3.
 * 
 * NOTE: Although this interface has no direct implementors, it's required for backwards compatibility.
 * 
 * @author chris
 */
public interface TopupAccountOnDemandPlugin extends Plugin {
    
    /**
     * Denotes the types of PaperCut account.
     * 
     * @author chris
     */
    enum AccountType {
        UserAccount,
        SharedAccount
    }
    
    /**
     * Class used to host the response from the <code>getTopup</code> method below.
     * 
     * @author chris
     */
    class TopupResponse {
        
        private double _topupAmount;
        private String _transactionComment;
        
        /**
         * @param topupAmount
         * @param transactionComment
         */
        public TopupResponse(double topupAmount, String transactionComment) {
            super();
            _topupAmount = topupAmount;
            _transactionComment = transactionComment;
        }
        
        /**
         * @return the topupAmount
         */
        public double getTopupAmount() {
            return _topupAmount;
        }
        /**
         * @param topupAmount the topupAmount to set
         */
        public void setTopupAmount(double topupAmount) {
            _topupAmount = topupAmount;
        }
        /**
         * @return the transactionComment
         */
        public String getTransactionComment() {
            return _transactionComment;
        }
        /**
         * @param transactionComment the transactionComment to set
         */
        public void setTransactionComment(String transactionComment) {
            _transactionComment = transactionComment;
        }
        

    }
    
    /**
     * This method is called when a usage request is received (e.g. a print job) and the target account does not
     * have enough funds.  The called method should perform a debit on the external source and return the topup amount.
     * 
     * @param amountRequired The amount required to perform the usage.  The topup value should be greater than this amount.
     * @param username The username performing the usage.
     * @param sharedAccount The shared account name associated with the usage, or empty/null if its only associated
     *                          with the user account (see username).
     * @param accountCharged The account that will be charged (e.g. the user or the shared account). 
     * @param usageDetails  A Map of key value pairs denoting the usage.  At the moment these are:
     *                  Print Job:
     *                          usage-type = "print"
     *                          printer-name
     *                          server-name
     *                          document-name
     *                          pages
     *                          
     * @return Null if adjustment is not to be performed, or a <code>TopupResponse</code> if external debit occurred.
     */
    TopupResponse getTopup(
            double amountRequired, 
            String username, 
            String sharedAccount, 
            AccountType accountCharged, 
            Map<String, String> usageDetails);


}
